@@ -20,9 +20,11 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
+#undef USE_OPTION_WIDGET
+
 #include <string.h>
 #include <glib.h>
 #include <gnome.h>
@@ -51,13 +53,15 @@ static const struct {
 } pix[] = {
   { "cd",         GTK_STOCK_CDROM       },
   { "line",       GNOME_STOCK_LINE_IN   },
-  { "microphone", GNOME_STOCK_MIC       },
-  { "mixer",      GST_MIXER_STOCK_MIXER },
+  { "mic",        GNOME_STOCK_MIC       },
+  { "mix",        GST_MIXER_STOCK_MIXER },
   { "pcm",        GST_MIXER_STOCK_TONE  },
+  { "headphone",  NULL                  },
   { "phone",      GST_MIXER_STOCK_PHONE },
   { "speaker",    GNOME_STOCK_VOLUME    },
   { "video",      GST_MIXER_STOCK_VIDEO },
   { "volume",     GST_MIXER_STOCK_TONE  },
+  { "master",     GST_MIXER_STOCK_TONE  },
   { NULL, NULL }
 };
 
@@ -138,27 +142,42 @@ cb_record_toggled (GtkWidget *button,
 		        gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
 }
 
+#if USE_OPTION_WIDGET
 static void
-create_track_widget (GstMixer      *mixer,
+cb_opt_changed (GtkComboBox *box,
+		gpointer     data)
+{
+  MyMixerControls *ctrl = (MyMixerControls *) data;
+  GtkTreeIter iter;
+  GtkTreeModel *model;
+  gchar *opt;
+
+  if (gtk_combo_box_get_active_iter (box, &iter)) {
+    model = gtk_combo_box_get_model (box);
+    gtk_tree_model_get (model, &iter, 0, &opt, -1);
+    gst_mixer_set_option (ctrl->mixer, GST_MIXER_OPTIONS (ctrl->track), opt);
+  }
+}
+#endif
+
+static MyMixerControls *
+create_track_header (GstMixer      *mixer,
 		     GtkWidget     *table,
 		     gint           column_pos,
 		     GstMixerTrack *track)
 {
-  GtkWidget *label, *slider, *button, *image;
-  GtkObject *adj;
-  gint i, *volumes;
-  GList *adjlist = NULL;
+  GtkWidget *label, *image;
+  gint i, c = track->num_channels ? track->num_channels : 1;
   MyMixerControls *ctrl = g_new0 (MyMixerControls, 1);
   gchar *str = NULL;
-
-  volumes = g_malloc (sizeof (gint) * track->num_channels);
+  gboolean found = FALSE;
 
   ctrl->mixer = mixer;
   ctrl->track = track;
   ctrl->locked = FALSE;
 
   /* image (optional) */
-  for (i = 0; str == NULL && pix[i].label != NULL; i++) {
+  for (i = 0; !found && pix[i].label != NULL; i++) {
     /* we dup the string to make the comparison case-insensitive */
     gchar *label_l = g_strdup (track->label),
 	  *needle_l = g_strdup (pix[i].label);
@@ -170,8 +189,10 @@ create_track_widget (GstMixer      *mixer,
     for (pos = 0; needle_l[pos] != '\0'; pos++)
       needle_l[pos] = g_ascii_tolower (needle_l[pos]);
 
-    if (g_strrstr (label_l, needle_l) != NULL)
+    if (g_strrstr (label_l, needle_l) != NULL) {
       str = pix[i].pixmap;
+      found = TRUE;
+    }
 
     g_free (label_l);
     g_free (needle_l);
@@ -180,9 +201,9 @@ create_track_widget (GstMixer      *mixer,
   if (str != NULL) {
     if ((image = gtk_image_new_from_stock (str, GTK_ICON_SIZE_MENU)) != NULL) {
       gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.5);
-      gtk_table_attach_defaults (GTK_TABLE (table), image,
-				 column_pos, column_pos + track->num_channels,
-				 0, 1);
+      gtk_table_attach (GTK_TABLE (table), image,
+			column_pos, column_pos + c,
+			0, 1, 5, 0, 0, 0);
     }
   }
 
@@ -190,11 +211,32 @@ create_track_widget (GstMixer      *mixer,
   label = gtk_label_new (track->label);
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
   gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
-  gtk_table_attach_defaults (GTK_TABLE (table), label,
-			     column_pos, column_pos + track->num_channels,
-			     1, 2);
+  gtk_table_attach (GTK_TABLE (table), label,
+		    column_pos, column_pos + c,
+		    1, 2, 5, 0, 0, 0);
+
+  return ctrl;
+}
+
+static void
+create_track_widget (GstMixer      *mixer,
+                     GtkWidget     *table,
+                     gint           column_pos,
+                     GstMixerTrack *track)
+{
+  GtkWidget *slider, *button;
+  gint i, *volumes;
+  GList *adjlist = NULL;
+  GtkObject *adj;
+  AtkObject *accessible;
+  gchar *accessible_name;
+  MyMixerControls *ctrl;
+
+  /* header */
+  ctrl = create_track_header (mixer, table, column_pos, track);
 
   /* now sliders for each of the tracks */
+  volumes = g_malloc (sizeof (gint) * track->num_channels);
   gst_mixer_get_volume (mixer, track, volumes);
   for (i = 0; i < track->num_channels; i++) {
     adj = gtk_adjustment_new (volumes[i],
@@ -210,6 +252,19 @@ create_track_widget (GstMixer      *mixer,
     gtk_table_attach_defaults (GTK_TABLE (table), slider,
 			       column_pos + i, column_pos + i + 1,
 			       2, 3);
+    accessible = gtk_widget_get_accessible (slider);
+    if (GTK_IS_ACCESSIBLE (accessible)) {
+       if (track->num_channels == 1)
+          accessible_name = g_strdup_printf (_("%s Slider"), ctrl->track->label);
+       else {
+          gchar *accessible_desc = g_strdup_printf (_("Channel %d of %s Slider"), i+1, ctrl->track->label);
+          accessible_name = g_strdup_printf (_("%s Slider %d"), ctrl->track->label, i+1);
+          atk_object_set_description (accessible, accessible_desc); 
+          g_free (accessible_desc);
+       }
+       atk_object_set_name (accessible, accessible_name);
+       g_free (accessible_name);
+    }
   }
 
   ctrl->adjustments = adjlist;
@@ -217,46 +272,144 @@ create_track_widget (GstMixer      *mixer,
   /* buttons (lock, mute, rec) */
   if (track->num_channels > 1) {
     button = gtk_check_button_new_with_label (_("Lock"));
-    gtk_table_attach_defaults (GTK_TABLE (table), button,
-			       column_pos, column_pos + track->num_channels,
-			       3, 4);
+    gtk_table_attach (GTK_TABLE (table), button,
+		      column_pos, column_pos + track->num_channels,
+		      3, 4, 5, 0, 0, 0);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
     ctrl->lock = button;
     g_signal_connect (G_OBJECT (button), "toggled",
 		      G_CALLBACK (cb_lock_toggled), (gpointer) ctrl);
+    accessible = gtk_widget_get_accessible (ctrl->lock);
+    if (GTK_IS_ACCESSIBLE (accessible)) {
+      accessible_name = g_strdup_printf (_("%s Lock"), ctrl->track->label);
+      atk_object_set_name (accessible, accessible_name);
+      g_free (accessible_name);
+    }
   }
 
   button = gtk_check_button_new_with_label (_("Mute"));
-  gtk_table_attach_defaults (GTK_TABLE (table), button,
-			     column_pos, column_pos + track->num_channels,
-			     4, 5);
+  gtk_table_attach (GTK_TABLE (table), button,
+		    column_pos, column_pos + track->num_channels,
+		    4, 5, 5, 0, 0, 0);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				GST_MIXER_TRACK_HAS_FLAG (track,
 						GST_MIXER_TRACK_MUTE));
   g_signal_connect (G_OBJECT (button), "toggled",
 		    G_CALLBACK (cb_mute_toggled), (gpointer) ctrl);
   ctrl->mute = button;
+  accessible = gtk_widget_get_accessible (ctrl->mute);
+  if (GTK_IS_ACCESSIBLE (accessible)) {
+    accessible_name = g_strdup_printf (_("%s Mute"), ctrl->track->label);
+    atk_object_set_name (accessible, accessible_name);
+    g_free (accessible_name);
+  }
 
   if (GST_MIXER_TRACK_HAS_FLAG (track, GST_MIXER_TRACK_INPUT)) {
     button = gtk_check_button_new_with_label (_("Rec."));
-    gtk_table_attach_defaults (GTK_TABLE (table), button,
-			       column_pos, column_pos + track->num_channels,
-			       5, 6);
+    gtk_table_attach (GTK_TABLE (table), button,
+		      column_pos, column_pos + track->num_channels,
+		      5, 6, 5, 0, 0, 0);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				  GST_MIXER_TRACK_HAS_FLAG (track,
 						GST_MIXER_TRACK_RECORD));
     g_signal_connect (G_OBJECT (button), "toggled",
 		      G_CALLBACK (cb_record_toggled), (gpointer) ctrl);
     ctrl->record = button;
+    accessible = gtk_widget_get_accessible (ctrl->record);
+    if (GTK_IS_ACCESSIBLE (accessible)) {
+      accessible_name = g_strdup_printf (_("%s Record"), ctrl->track->label);
+      atk_object_set_name (accessible, accessible_name);
+      g_free (accessible_name);
+    }
   }
-
   g_free (volumes);
 }
 
+#if USE_OPTION_WIDGET
+static void
+create_options_widget (GstMixer      *mixer,
+		       GtkWidget     *table,
+		       gint           column_pos,
+		       GstMixerTrack *track)
+{
+  GtkWidget *box;
+  GstMixerOptions *options = GST_MIXER_OPTIONS (track);
+  MyMixerControls *ctrl;
+  const GList *opt;
+  AtkObject *accessible;
+  gchar *accessible_name;
+  gint i = 0;
+  const gchar *active_opt;
+
+  /* header */
+  ctrl = create_track_header (mixer, table, column_pos, track);
+
+  /* optionmenu */
+  active_opt = gst_mixer_get_option (mixer, GST_MIXER_OPTIONS (track));
+  box = gtk_combo_box_new_text ();
+  for (opt = options->values; opt != NULL; opt = opt->next, i++) {
+    gtk_combo_box_append_text (GTK_COMBO_BOX (box), opt->data);
+    if (!strcmp (active_opt, opt->data)) {
+      gtk_combo_box_set_active (GTK_COMBO_BOX (box), i);
+    }
+  }
+  gtk_table_attach (GTK_TABLE (table), box, column_pos, column_pos + 1, 2, 3,
+		    0, 0, 0, 0);
+  accessible = gtk_widget_get_accessible (box);
+  if (GTK_IS_ACCESSIBLE (accessible)) {
+    /* once this code is un-#if 0'ed, please i18n'ize this */
+    accessible_name = g_strdup_printf ("%s Option Selection",
+				       ctrl->track->label);
+    atk_object_set_name (accessible, accessible_name);
+    g_free (accessible_name);
+  }
+  gtk_widget_show (box);
+  g_signal_connect (box, "changed", G_CALLBACK (cb_opt_changed), ctrl);
+}
+#endif
+
+static void
+create_switch_widget (GstMixer      *mixer,
+		      GtkWidget     *table,
+		      gint           column_pos,
+		      GstMixerTrack *track)
+{
+  GtkWidget *button;
+  MyMixerControls *ctrl;
+  AtkObject *accessible;
+  gchar *accessible_name;
+
+  /* header */
+  ctrl = create_track_header (mixer, table, column_pos, track);
+
+  /* mute button */
+  button = gtk_check_button_new_with_label (_("Mute"));
+  gtk_table_attach (GTK_TABLE (table), button,
+                    column_pos, column_pos + 1,
+                    4, 5, 5, 0, 0, 0);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
+                                GST_MIXER_TRACK_HAS_FLAG (track,
+                                                GST_MIXER_TRACK_MUTE));
+  g_signal_connect (G_OBJECT (button), "toggled",
+                    G_CALLBACK (cb_mute_toggled), (gpointer) ctrl);
+  ctrl->mute = button;
+  accessible = gtk_widget_get_accessible (ctrl->mute);
+  if (GTK_IS_ACCESSIBLE (accessible)) {
+    accessible_name = g_strdup_printf (_("%s Mute"), ctrl->track->label);
+    atk_object_set_name (accessible, accessible_name);
+    g_free (accessible_name);
+  }
+}
+
 static GtkWidget *
 create_mixer_widget (GstMixer *mixer)
 {
+  GtkWidget *view;
   GtkWidget *table;
+  GtkWidget *viewport;
+  GtkAdjustment *hadjustment;
+  GtkAdjustment *vadjustment;
+
   gint tablepos = 0;
   const GList *tracks;
 
@@ -278,9 +431,27 @@ create_mixer_widget (GstMixer *mixer)
   tablepos = 0;
   tracks = gst_mixer_list_tracks (mixer);
   for ( ; tracks != NULL; tracks = tracks->next) {
-    create_track_widget (mixer, table, tablepos,
-			 (GstMixerTrack *) tracks->data);
-    tablepos += ((GstMixerTrack *) tracks->data)->num_channels;
+    GstMixerTrack *track = GST_MIXER_TRACK (tracks->data);
+
+    /* hack for bad API (because of API stability...) in GStreamer-0.8.x */
+    if (track->num_channels == 0) {
+      if (GST_IS_MIXER_OPTIONS (track)) {
+#if USE_OPTION_WIDGET
+        /* options - menu */
+        create_options_widget (mixer, table, tablepos, track);
+#else
+        /* ignore */
+        continue;
+#endif
+      } else {
+        /* switch */
+        create_switch_widget (mixer, table, tablepos, track);
+      }
+    } else {
+      create_track_widget (mixer, table, tablepos, track);
+    }
+
+    tablepos += track->num_channels ? track->num_channels : 1;
     if (tracks->next != NULL) {
       GtkWidget *sep = gtk_vseparator_new ();
       gtk_table_attach_defaults (GTK_TABLE (table), sep,
@@ -289,7 +460,26 @@ create_mixer_widget (GstMixer *mixer)
     }
   }
 
-  return table;
+  /* put table in a scrollview for handling lots of tracks */
+  view = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (view),
+				  tablepos <= 30 ?
+				  GTK_POLICY_NEVER : GTK_POLICY_AUTOMATIC,
+				  GTK_POLICY_NEVER);
+  if (tablepos > 30)
+    gtk_widget_set_size_request (view, 600, -1);
+
+  hadjustment = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW (view));
+  vadjustment = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW (view));
+
+  viewport = gtk_viewport_new(GTK_ADJUSTMENT (hadjustment), GTK_ADJUSTMENT (vadjustment));
+  gtk_viewport_set_shadow_type(GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);
+
+  gtk_container_add(GTK_CONTAINER (viewport), table);
+  gtk_container_add(GTK_CONTAINER (view), viewport);
+
+
+  return view;
 }
 
 static GList *
@@ -330,7 +520,8 @@ create_mixer_collection (GtkWidget *notebook)
     probe = GST_PROPERTY_PROBE (element);
     if (!(devspec = gst_property_probe_get_property (probe, "device")))
       goto next;
-    array = gst_property_probe_probe_and_get_values (probe, devspec);
+    if (!(array = gst_property_probe_probe_and_get_values (probe, devspec)))
+      goto next;
 
     /* set all devices and test for mixer */
     for (n = 0; n < array->n_values; n++) {
@@ -389,13 +580,16 @@ cb_about (GtkWidget *widget,
 {
   GtkWidget *about;
   const gchar *authors[] = { "Ronald Bultje <rbultje@ronald.bitfreak.net>",
+			     "Leif Johnson <leif@ambient.2y.net>",
 			     NULL };
+  const gchar *documentors[] = { "Sun Microsystems",
+				 NULL};
 
-  about = gnome_about_new (_("GStreamer Volume Control"),
+  about = gnome_about_new (_("Volume Control"),
 			   VERSION,
-			   "(c) 2003 Ronald Bultje",
+			   "(c) 2003-2004 Ronald Bultje",
 			   _("A GNOME/GStreamer-based mixer application"),
-			   authors, NULL, NULL,
+			   authors, documentors, NULL,
 			   NULL);
 
   gtk_widget_show (about);
@@ -414,8 +608,8 @@ static GnomeUIInfo file_menu[] = {
 };
 
 static GnomeUIInfo help_menu[] = {
+  GNOMEUIINFO_HELP ("gnome-volume-control"),
   GNOMEUIINFO_MENU_ABOUT_ITEM (cb_about, NULL),
-  GNOMEUIINFO_HELP (PACKAGE),
   GNOMEUIINFO_END
 };
 
@@ -469,9 +663,14 @@ main (gint   argc,
     POPT_TABLEEND
   };
 
+  /* yay! */
+  bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+  textdomain (GETTEXT_PACKAGE);
+
   /* init gstreamer/gnome */
   options[0].arg = (void *) gst_init_get_popt_table();
-  gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
+  gnome_program_init ("gnome-volume-control", VERSION, LIBGNOMEUI_MODULE,
 		      argc, argv,
 		      GNOME_PARAM_POPT_TABLE, options,
 		      GNOME_PARAM_APP_DATADIR, DATA_DIR,
@@ -479,7 +678,7 @@ main (gint   argc,
   register_stock_icons ();
 
   /* create main window + menus */
-  window = gnome_app_new (PACKAGE, _("GStreamer Volume Control"));
+  window = gnome_app_new ("gnome-volume-control", _("Volume Control"));
   gnome_app_create_menus (GNOME_APP (window), main_menu);
  
   /* Set appicon image */ 
